// Read an integer from Serial input
int get_number() {

  // Wait until input is available
  while (Serial.available() <= 0)
    delay(1);

  // Read into buffer
  const int SIZE = 200;
  char buffer[SIZE];
  int i = 0;

  while (Serial.available() > 0 && i < SIZE) {
    int byte = Serial.read();    
    if (byte < 0)
      break;
    buffer[i++] = (char)byte;
  }
  buffer[i] = '\0';
  int x = atoi(buffer);
  Serial.println(x);  // Simple echo
  return x;
}

int get_number_of_values() {
  Serial.print("Number of values: ");
  return get_number();
}

int get_value(int i, int n) {
  Serial.print("Value ");
  Serial.print(i);
  Serial.print("/");
  Serial.print(n);
  Serial.print(": ");
  return get_number();
}

// Compute average
double avg(int x[], int n)
{
  double sum = 0;
  for (int i = 0; i < n; i++)
     sum += x[i];
  return sum / n;
}

// Sort
void insertion_sort(int a[], int size){
    for (int i = 1; i < size; i++) {
        int j = i;
        while (j > 0 && a[j - 1] > a[j]) {
            // printf("Swap %d and %d\n", a[j], a[j - 1]);
            int tmp = a[j];
            a[j] = a[j - 1];
            a[j - 1] = tmp;
            j--;
        }
    }
}


// Compute median
double med(int x[], int n)
{
  // Copy array such that we can sort it
  int *a = (int *)malloc(sizeof(int) * n);
  for (int i = 0; i < n; i++)
    a[i] = x[i];
  
  insertion_sort(a, n);
  
  double median = (x[n / 2] + x[(n - 1) / 2]) * 0.5;
  
  free(a);
  return median;
}

void compute_average() {
  int n_values = get_number_of_values();
  int *values = (int *)malloc(sizeof(int) * n_values);
  for (int i = 0; i < n_values; i++)
    values[i] = get_value(i + 1, n_values);
  
  /*
  for (int i = 0; i < n_values; i++) {
    Serial.print("values[");
    Serial.print(i);
    Serial.print("] = ");
    Serial.println(values[i]);
  }
  */
  
  double average = avg(values, n_values);
  double median  = med(values, n_values);

  Serial.print("Average:");
  Serial.print(average);
  Serial.print(" Median:");
  Serial.print(median);
  Serial.println("");
 
  free(values); 
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
}

void loop() {
  // put your main code here, to run repeatedly:  
  compute_average();
}
